//
//  EHHttpRequest.h
//  EweiSDK
//
//  Created by Kiwi on 2018/1/18.
//  Copyright © 2018年 infocare. All rights reserved.
//

#import <Foundation/Foundation.h>

@class EHTicket, EHArticle, EHArticleTopic;

/*
 EHHttpRequest 请求执行条件：
 
 - 请求结果将通过 block 异步返回；
 - 必须在 EHSDKManager 调用登录授权之后才能使用；
 
 
 EHHttpRequest 部分参数说明：
 @param <NSInteger> page 当前页码，从 1 开始，非必传参数
 @param <NSInteger> count 单页数据量，非必传参数
 @param <block> <NSArray*> result 为请求返回结果，<NSError*> error 为错误信息，请求成功时为 nil
 */



@interface EHHttpRequest : NSObject

#pragma mark API - 工单

/*
 获取工单列表接口，以下 4 个方法请求依次是“我的工单”、“抄送给我的工单”、“我所在组的工单”、“待评价的工单”
 */

/* 我的工单 */
+ (void)requestMyTicketsWithPage:(NSInteger)page
                           count:(NSInteger)count
                      completion:(void(^) (NSArray <EHTicket*> * result, NSError * error))completionBlock;


/* 抄送给我的工单 */
+ (void)requestCCSTicketsWithPage:(NSInteger)page
                            count:(NSInteger)count
                       completion:(void(^) (NSArray <EHTicket*> * result, NSError * error))completionBlock;


/* 我所在组的工单 */
+ (void)requestMyGroupTicketsWithPage:(NSInteger)page
                                count:(NSInteger)count
                           completion:(void(^) (NSArray <EHTicket*> * result, NSError * error))completionBlock;


/* 待评价的工单 */
+ (void)requestUnevaluatedTicketsWithPage:(NSInteger)page
                                    count:(NSInteger)count
                               completion:(void(^) (NSArray <EHTicket*> * result, NSError * error))completionBlock;





#pragma mark API - 文章

/*
 获取推荐文章列表
 */
+ (void)requestRecommendedArticlesWithPage:(NSInteger)page
                                     count:(NSInteger)count
                                completion:(void(^) (NSArray <EHArticle*> * result, NSError * error))completionBlock;


/*
 通过关键字搜索文章
 
 @param <NSString*> keyword 搜索关键字，必传参数
 */
+ (void)requestArticlesWithSearchKeyword:(NSString*)keyword
                                    page:(NSInteger)page
                                   count:(NSInteger)count
                              completion:(void(^) (NSArray <EHArticle*> * result, NSError * error))completionBlock;


/*
 获取所有文章目录及子目录
 */
+ (void)requestAllArticleTopicsWithCompletion:(void(^) (NSArray <EHArticleTopic*> * result, NSError * error))completionBlock;


/*
 获取指定目录下的文章
 
 @param <EHArticleTopic*> topic 文章目录，必传参数
 */
+ (void)requestArticlesWithTopic:(EHArticleTopic*)topic
                            page:(NSInteger)page
                           count:(NSInteger)count
                      completion:(void(^) (NSArray <EHArticle*> * result, NSError * error))completionBlock;





#pragma mark API - 论坛、帖子

/*
 获取推荐帖子列表
 */
+ (void)requestRecommendedForumArticlesWithPage:(NSInteger)page
                                          count:(NSInteger)count
                                     completion:(void(^) (NSArray <EHArticle*> * result, NSError * error))completionBlock;


/*
 获取热门帖子列表
 */
+ (void)requestHotForumArticlesWithPage:(NSInteger)page
                                  count:(NSInteger)count
                             completion:(void(^) (NSArray <EHArticle*> * result, NSError * error))completionBlock;


/*
 通过关键字搜索帖子
 
 @param <NSString*> keyword 搜索关键字，必传参数
 */
+ (void)requestForumArticlesWithSearchKeyword:(NSString*)keyword
                                         page:(NSInteger)page
                                        count:(NSInteger)count
                                   completion:(void(^) (NSArray <EHArticle*> * result, NSError * error))completionBlock;


/*
 获取所有论坛目录及子目录
 */
+ (void)requestAllForumArticleTopicsWithCompletion:(void(^) (NSArray <EHArticleTopic*> * result, NSError * error))completionBlock;


/*
 获取指定目录下的帖子
 
 @param <EHArticleTopic*> topic 论坛目录，必传参数
 */
+ (void)requestForumArticlesWithTopic:(EHArticleTopic*)topic
                                 page:(NSInteger)page
                                count:(NSInteger)count
                           completion:(void(^) (NSArray <EHArticle*> * result, NSError * error))completionBlock;

@end
